/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.io.gff;

import jalview.bin.ApplicationSingletonProvider;
import jalview.bin.ApplicationSingletonProvider.ApplicationSingletonI;

/**
 * A factory class that returns a model of the Sequence Ontology. By default a
 * hard-coded subset is used (for the applet, or testing), or
 * setSequenceOntology() can be used to set full Ontology data.
 * 
 * @author gmcarstairs
 *
 */
public class SequenceOntologyFactory implements ApplicationSingletonI
{
  /**
   * Answers an instance of this class for the current application context. Note
   * that this supports running two JS 'applets' on the same page, one with the
   * full Sequence Ontology (USE_FULL_SO = true) and one with a hard-coded
   * subset (USE_FULL_SO = false). If this is overkill, could change this method
   * to just return a common static instance.
   * 
   * @return
   */
  private static synchronized SequenceOntologyFactory getInstance()
  {
    return (SequenceOntologyFactory) ApplicationSingletonProvider
            .getInstance(SequenceOntologyFactory.class);
  }

  /**
   * Answers the configured model of the Sequence Ontology.
   * 
   * @return
   */
  public static synchronized SequenceOntologyI getSequenceOntology()
  {
    SequenceOntologyFactory f = getInstance();
    return (f.sequenceOntology == null
            ? f.sequenceOntology = new SequenceOntologyLite()
            : f.sequenceOntology);
  }

  /**
   * For testng only
   * 
   * @param so
   */
  public static void setSequenceOntology(SequenceOntologyI so)
  {
    getInstance().sequenceOntology = so;
  }

  private SequenceOntologyI sequenceOntology;

  private SequenceOntologyFactory()
  {
    // private singleton
  }

}
